#version 330
#extension GL_ARB_separate_shader_objects : enable





layout(location = 0) smooth in vec2 _fin_fragTexCoord;

struct SFragmentInput
{
	vec2 fragTexCoord;
	bool IsFrontFace;
};

layout(location = 0) out vec4 _fout_outColor;

struct SFragmentOutput
{
	vec4 outColor;
};

 uniform sampler2D Texture;
 uniform sampler2D LookUp;

layout( std140) uniform UColorRemapInfo
{
	float LutHeight;
} ColorRemapInfo;



vec2 ColorToUV(float r, float g, float b, float lutHeight)
{
    return vec2( (b + (r + 0.5f)/lutHeight) / (lutHeight), (g + 0.5f)/lutHeight);
}

void FragmentMain( in SFragmentInput  fInput,  out SFragmentOutput  fOutput  )
{
    vec4 fullscreenColor =  texture(Texture, fInput.fragTexCoord) ;
    float lutHeight =  ColorRemapInfo.LutHeight ;

    vec3 texelCoord = fullscreenColor.rgb * (lutHeight - 1.0f);
    vec3 coord0 = floor(texelCoord);
    vec3 coord1 = coord0 + 1.0f;
    vec3 coordFrac = texelCoord - coord0;

    vec4 col_000 =  texture(LookUp, ColorToUV(coord0.r, coord0.g, coord0.b, lutHeight)) ;
    vec4 col_100 =  texture(LookUp, ColorToUV(coord1.r, coord0.g, coord0.b, lutHeight)) ;
    vec4 col_010 =  texture(LookUp, ColorToUV(coord0.r, coord1.g, coord0.b, lutHeight)) ;
    vec4 col_110 =  texture(LookUp, ColorToUV(coord1.r, coord1.g, coord0.b, lutHeight)) ;
    vec4 col_001 =  texture(LookUp, ColorToUV(coord0.r, coord0.g, coord1.b, lutHeight)) ;
    vec4 col_101 =  texture(LookUp, ColorToUV(coord1.r, coord0.g, coord1.b, lutHeight)) ;
    vec4 col_011 =  texture(LookUp, ColorToUV(coord0.r, coord1.g, coord1.b, lutHeight)) ;
    vec4 col_111 =  texture(LookUp, ColorToUV(coord1.r, coord1.g, coord1.b, lutHeight)) ;

    vec4 col_00 = mix(col_000, col_100, coordFrac.r);
    vec4 col_10 = mix(col_010, col_110, coordFrac.r);
    vec4 col_01 = mix(col_001, col_101, coordFrac.r);
    vec4 col_11 = mix(col_011, col_111, coordFrac.r);

    vec4 col_0 = mix(col_00, col_10, coordFrac.g);
    vec4 col_1 = mix(col_01, col_11, coordFrac.g);

    vec4 col = mix(col_0, col_1, coordFrac.b);
    fOutput.outColor = vec4(col.rgb, fullscreenColor.a);
}

void main()
{
	SFragmentInput fInput;
	SFragmentOutput fOutput;

	fInput.fragTexCoord = _fin_fragTexCoord;
	fInput.IsFrontFace = gl_FrontFacing;
	FragmentMain(fInput, fOutput);
	_fout_outColor = fOutput.outColor;
}
