#version 430
#extension GL_ARB_separate_shader_objects : enable





layout(location = 0) in vec2 _vin_TexCoords;

struct SVertexInput
{
	vec2 TexCoords;
	int VertexIndex;
	uint InstanceId;
};

layout(location = 0) smooth out vec2 _vout_fragTexCoords;
layout(location = 1) flat out vec4 _vout_fragColor;
layout(location = 2) flat out uint _vout_fragFlags;
layout(location = 3) flat out uvec4 _vout_fragDrawDigitCounts;
layout(location = 4) flat out vec4 _vout_fragValue;

struct SVertexOutput
{
	vec2 fragTexCoords;
	vec4 fragColor;
	uint fragFlags;
	uvec4 fragDrawDigitCounts;
	vec4 fragValue;
	vec4 VertexPosition;
};

layout( std140) uniform USceneInfo
{
	mat4 ViewProj;
	mat4 View;
	mat4 Proj;
	mat4 InvView;
	mat4 InvViewProj;
	mat4 UserToLHZ;
	mat4 LHZToUser;
	mat4 BillboardingView;
	mat4 PackNormalView;
	mat4 UnpackNormalView;
	vec4 SideVector;
	vec4 DepthVector;
	vec2 ZBufferLimits;
	vec2 ViewportSize;
	float Handedness;
} SceneInfo;
layout(std430 ) readonly buffer BData
{
	uint Data[];
} Data;
layout(std430 ) readonly buffer BAtlas
{
	uint Atlas[];
} Atlas;



mat4 GetMeshMatrix( in SVertexInput  vInput  )
{
	return  mat4(vec4(1, 0, 0, 0), vec4(0, 1, 0, 0), vec4(0, 0, 1, 0), vec4(0, 0, 0, 1)) ;

}





uint DigitCount(float absValue, uint comma, bool isFloat)
{

	const vec4 absValue4 = vec4(absValue, absValue, absValue, absValue);
	const vec4 digitCountSelector = vec4( greaterThanEqual(absValue4, vec4(-1, 10, 100, 1000)) );
	const uint digitCount = uint(dot(digitCountSelector, digitCountSelector));

	const uint staticChar = isFloat ? 3u : 0u;

	return staticChar +
			1 +
			digitCount +
			comma;
}

void VertexMain( in SVertexInput  vInput,  out SVertexOutput  vOutput  )
{
	const uint particleID = vInput.InstanceId;


	const vec3 worldPos =  vec3( uintBitsToFloat(Data.Data [ (particleID * 16 + 0) ]) , uintBitsToFloat(Data.Data [( (particleID * 16 + 0) ) + 1]) , uintBitsToFloat(Data.Data [( (particleID * 16 + 0) ) + 2]) ) ;
	const float fontScale =  uintBitsToFloat(Data.Data [ (particleID * 16 + 3) ]) ;
	const vec4 color =  vec4( uintBitsToFloat(Data.Data [ (particleID * 16 + 4) ]) , uintBitsToFloat(Data.Data [( (particleID * 16 + 4) ) + 1]) , uintBitsToFloat(Data.Data [( (particleID * 16 + 4) ) + 2]) , uintBitsToFloat(Data.Data [( (particleID * 16 + 4) ) + 3]) ) ;
	const vec2 screenOffset =  vec2( uintBitsToFloat(Data.Data [ (particleID * 16 + 8) ]) , uintBitsToFloat(Data.Data [( (particleID * 16 + 8) ) + 1]) ) ;
	const uint flags =  Data.Data [ (particleID * 16 + 10) ] ;
	const vec4 valuesF =  vec4( uintBitsToFloat(Data.Data [ (particleID * 16 + 12) ]) , uintBitsToFloat(Data.Data [( (particleID * 16 + 12) ) + 1]) , uintBitsToFloat(Data.Data [( (particleID * 16 + 12) ) + 2]) , uintBitsToFloat(Data.Data [( (particleID * 16 + 12) ) + 3]) ) ;
	const ivec4 valuesI =  ivec4( Data.Data [ (particleID * 16 + 12) ], Data.Data [( (particleID * 16 + 12) ) + 1], Data.Data [( (particleID * 16 + 12) ) + 2], Data.Data [( (particleID * 16 + 12) ) + 3]) ;

	const uint alignment = flags & 0xFF;
	const uint drawDim = (flags >> 8) & 0xFF;
	const bool isFloat = (flags & 0x010000) != 0;





	const float kAlignRemap[3] = { -1.0f, 0.0f, 1.0f };


	const vec4 drawDim4 = vec4(drawDim, drawDim, drawDim, drawDim);
	const vec4 activeDims = vec4( greaterThan(drawDim4, vec4(0,1,2,3)) );


	const vec4 values = isFloat ? valuesF : vec4(valuesI);
	const vec4 absValues = abs(values) + 0.0005f;

	vOutput.fragValue = values;


	vOutput.fragDrawDigitCounts.x = uint(DigitCount(absValues.x, uint(activeDims.y), isFloat) * activeDims.x);
	vOutput.fragDrawDigitCounts.y = uint(DigitCount(absValues.y, uint(activeDims.z), isFloat) * activeDims.y);
	vOutput.fragDrawDigitCounts.z = uint(DigitCount(absValues.z, uint(activeDims.w), isFloat) * activeDims.z);
	vOutput.fragDrawDigitCounts.w = uint(DigitCount(absValues.w, 0u, isFloat) * activeDims.w);

	const float charSize =  0.05f  * fontScale;
	const uint digitCount = uint(dot(vec4(vOutput.fragDrawDigitCounts), activeDims));
	const vec2 radius = vec2(charSize * digitCount, charSize *  2.0f );

	const vec3 tangent0 =  ( SceneInfo.BillboardingView )[0] .xyz * radius.x;
	const vec3 tangent1 =  ( SceneInfo.BillboardingView )[1] .xyz * radius.y * -1.0f;


	const float alignmentUVs = vInput.TexCoords.x + kAlignRemap[alignment];
	const vec3 bbCorner = tangent0 * alignmentUVs + tangent1 * vInput.TexCoords.y;


	const float sideScreenOffset = screenOffset.x - (charSize * float(vOutput.fragValue.x >= 0));

	const vec3 vertexWorldOffset =  ( SceneInfo.BillboardingView )[0] .xyz * sideScreenOffset +
									( SceneInfo.BillboardingView )[1] .xyz * screenOffset.y;

	const vec3 vertexWorldPosition = worldPos + bbCorner + vertexWorldOffset;

	vOutput.VertexPosition =  (( SceneInfo.ViewProj ) * (vec4(vertexWorldPosition, 1.0))) ;
	vOutput.fragTexCoords = vInput.TexCoords * 0.5f + 0.5f;
	vOutput.fragColor = color;
	vOutput.fragFlags = flags;
}


void main()
{
	SVertexInput vInput;
	SVertexOutput vOutput;

	vInput.TexCoords = _vin_TexCoords;
	vInput.VertexIndex = gl_VertexID;
	vInput.InstanceId = uint(gl_InstanceID);
	vOutput.fragTexCoords = vec2(0, 0);
	vOutput.fragColor = vec4(0, 0, 0, 0);
	vOutput.fragFlags = uint(0);
	vOutput.fragDrawDigitCounts = uvec4(0, 0, 0, 0);
	vOutput.fragValue = vec4(0, 0, 0, 0);
	VertexMain(vInput, vOutput);
	_vout_fragTexCoords = vOutput.fragTexCoords;
	_vout_fragColor = vOutput.fragColor;
	_vout_fragFlags = vOutput.fragFlags;
	_vout_fragDrawDigitCounts = vOutput.fragDrawDigitCounts;
	_vout_fragValue = vOutput.fragValue;
	gl_Position = vOutput.VertexPosition;
}
